﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.Paint;
using Telerik.WinControls.Primitives;
using Telerik.WinControls.UI;

namespace _1037352_RadGridViewPrinting
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            this.radGridView1.DataSource = this.GetData();
            this.radGridView1.AutoSizeColumnsMode = Telerik.WinControls.UI.GridViewAutoSizeColumnsMode.Fill;
            this.radGridView1.Columns[0].DisableHTMLRendering = false;

            this.radGridView1.PrintCellFormatting += radGridView1_PrintCellFormatting;
        }

        private void radGridView1_PrintCellFormatting(object sender, Telerik.WinControls.UI.PrintCellFormattingEventArgs e)
        {
            e.PrintCell.EnableHtmlTextRendering = true;
        }

        private DataTable GetData()
        {
            DataTable dt = new DataTable();

            dt.Columns.Add("Name", typeof(string));
            dt.Columns.Add("Date", typeof(DateTime));
            dt.Columns.Add("Bool", typeof(bool));
            dt.Columns.Add("Description", typeof(string));
            for (int i = 0; i < 10; i++)
            {
                for (int j = 0; j < 5; j++)
                {
                    dt.Rows.Add(@"<html><b>Name " + j, DateTime.Now.AddDays(i), i % 2 == 0 ? true : false, "Description " + i);
                }
            }

            return dt;
        }

        private void radButton1_Click(object sender, EventArgs e)
        {
            MyGridPrintStyle style = new MyGridPrintStyle();
            style.FitWidthMode = PrintFitWidthMode.FitPageWidth;
            style.PrintGrouping = true;
            style.PrintSummaries = false;

            style.PrintHeaderOnEachPage = true;
            style.PrintHiddenColumns = false;
            style.CellPadding = new Padding(100, 0, 0, 0);

            this.radGridView1.PrintStyle = style;
            this.radGridView1.PrintPreview();
        }
    }

    public class MyGridPrintStyle : GridPrintStyle
    {
        protected override BaseGridPrintRenderer InitializePrintRenderer(RadGridView grid)
        {
            if (this.PrintRenderer != null)
            {
                this.PrintRenderer.PrintCellPaint -= renderer_PrintCellPaint;
                this.PrintRenderer.PrintCellFormatting -= renderer_PrintCellFormatting;
                this.PrintRenderer.ChildViewPrinting -= renderer_ChildViewPrinting;
            }

            BaseGridPrintRenderer renderer = null;

            if (grid.ViewDefinition.GetType() == typeof(ColumnGroupsViewDefinition))
            {
                if (!(renderer is ColumnGroupsViewDefinitionPrintRenderer))
                {
                    renderer = new ColumnGroupsViewDefinitionPrintRenderer(grid);
                }
            }
            else if (grid.ViewDefinition.GetType() == typeof(HtmlViewDefinition))
            {
                if (!(renderer is HtmlViewDefinitionPrintRenderer))
                {
                    renderer = new HtmlViewDefinitionPrintRenderer(grid);
                }
            }
            else
            {
                if (!(renderer is TableViewDefinitionPrintRenderer))
                {
                    renderer = new MyTableViewDefinitionPrintRenderer(grid);
                }
            }

            renderer.ChildViewPrinting += renderer_ChildViewPrinting;
            renderer.PrintCellFormatting += renderer_PrintCellFormatting;
            renderer.PrintCellPaint += renderer_PrintCellPaint;

            return renderer;
        }

        private void renderer_PrintCellPaint(object sender, PrintCellPaintEventArgs e)
        {
            this.OnPrintCellPaint(sender, e);
        }

        private void renderer_PrintCellFormatting(object sender, PrintCellFormattingEventArgs e)
        {
            this.OnPrintCellFormatting(sender, e);
        }

        private void renderer_ChildViewPrinting(object sender, ChildViewPrintingEventArgs e)
        {
            this.OnChildViewPrinting(sender, e);
        }
    }

    public class MyTableViewDefinitionPrintRenderer : TableViewDefinitionPrintRenderer
    {
        public MyTableViewDefinitionPrintRenderer(RadGridView grid)
            : base(grid)
        { }

        protected override CellPrintElement CreateDataCellPrintElement(GridViewCellInfo cellInfo)
        {
            MyCellPrintElement cell;

            if (cellInfo.ColumnInfo is GridViewImageColumn)
            {
                cell = CreateImageCellPrintElement(cellInfo);
                cell.DrawText = false;
            }
            else
            {
                cell = new MyCellPrintElement();
            }

            if (this.GridView.PrintStyle.PrintAlternatingRowColor && this.GridView.PrintStyle.PrintTraverser.Current.Index % 2 == 1)
            {
                cell.BackColor = this.GridView.PrintStyle.AlternatingRowColor;
            }
            else
            {
                cell.BackColor = this.GridView.PrintStyle.CellBackColor;
            }

            cell.BorderColor = this.GridView.PrintStyle.BorderColor;

            if (cell.BackColor != null && cell.BackColor != Color.Transparent && cell.BackColor != Color.Empty)
            {
                cell.DrawFill = true;
            }

            GridViewDataColumn dataColumn = cellInfo.ColumnInfo as GridViewDataColumn;

            object value = cellInfo.Value;
            GridViewComboBoxColumn comboBoxColumn = cellInfo.ColumnInfo as GridViewComboBoxColumn;
            if (comboBoxColumn != null && comboBoxColumn.HasLookupValue)
            {
                value = comboBoxColumn.GetLookupValue(value);
            }

            if (dataColumn != null && !string.IsNullOrEmpty(dataColumn.FormatString))
            {
                if (cellInfo.ColumnInfo is GridViewComboBoxColumn)
                {
                    GridViewComboBoxColumn comboColumn = (GridViewComboBoxColumn)cellInfo.ColumnInfo;
                    Type comboColumnType = (Type)comboColumn.GetType().GetProperty("DisplayMemberDataType", BindingFlags.Instance | BindingFlags.NonPublic).GetValue(comboColumn);

                    dataColumn.DataType = comboColumnType;
                    dataColumn.DataTypeConverter = TypeDescriptor.GetConverter(comboColumnType);
                    cell.Text = RadDataConverter.Instance.Format(value, typeof(string), dataColumn) as string;
                }
                else
                {
                    cell.Text = RadDataConverter.Instance.Format(value, typeof(string), dataColumn) as string;
                }
            }
            else
            {
                cell.Text = (value != null) ? value.ToString() : String.Empty;
            }

            cell.TextAlignment = cellInfo.ColumnInfo.TextAlignment;
            cell.StringTrimming = StringTrimming.EllipsisCharacter;

            if (this.GridView.RightToLeft == RightToLeft.Yes)
            {
                cell.StringFormatFlags |= StringFormatFlags.DirectionRightToLeft;
            }

            if (!cellInfo.ColumnInfo.WrapText)
            {
                cell.StringFormatFlags |= StringFormatFlags.NoWrap;
            }

            return cell;
        }

        private MyCellPrintElement CreateImageCellPrintElement(GridViewCellInfo cellInfo)
        {
            MyCellPrintElement cell = new MyCellPrintElement();
            object val = cellInfo.Value;

            if (val == null || val == DBNull.Value)
            {
                cell.Image = null;
                return cell;
            }

            if (val is byte[])
            {
                cell.Image = ImageHelper.GetImageFromBytes((byte[])val);
            }
            else if (val is Image)
            {
                cell.Image = val as Image;
            }

            if (cell.Image != null)
            {
                GridViewImageColumn imageColumn = cellInfo.ColumnInfo as GridViewImageColumn;
                cell.ImageLayout = imageColumn != null ? imageColumn.ImageLayout : ImageLayout.Stretch;
                cell.ImageAlignment = imageColumn != null ? imageColumn.ImageAlignment : ContentAlignment.MiddleCenter;
            }

            return cell;
        }
    }

    public class MyCellPrintElement : CellPrintElement
    {
        protected override void PaintText(Graphics g, RectangleF rect)
        {
            Padding padding = this.TextPadding;

            if (String.IsNullOrEmpty(this.Text) || !this.DrawText)
            {
                return;
            }

            if (this.EnableHtmlTextRendering && this.Text.Contains("<html>"))
            {
                TextPrimitiveHtmlImpl impl = new TextPrimitiveHtmlImpl();
                RadGdiGraphics graphics = new RadGdiGraphics(g);

                TextParams tp = new TextParams();
                tp.alignment = this.TextAlignment;
                tp.ClipText = true;
                tp.font = this.Font;
                tp.foreColor = this.ForeColor;
                tp.paintingRectangle = rect;
                tp.rightToLeft = this.RightToLeft;
                tp.text = this.Text;
                tp.textWrap = true;

                if (padding != Padding.Empty)
                {
                    tp.paintingRectangle.X += padding.Left;
                    tp.paintingRectangle.Y += padding.Top;
                    tp.paintingRectangle.Width -= padding.Horizontal;
                    tp.paintingRectangle.Height -= padding.Vertical;
                }

                impl.PaintPrimitive(graphics, tp);

                return;
            }

            using (StringFormat format = new StringFormat())
            using (SolidBrush brush = new SolidBrush(this.ForeColor))
            {
                format.Alignment = this.GetHorizontalAlignment(this.TextAlignment);
                format.LineAlignment = this.GetVerticalAlignment(this.TextAlignment);
                format.Trimming = this.StringTrimming;
                format.FormatFlags = this.StringFormatFlags;

                if (padding != Padding.Empty)
                {
                    rect.X += padding.Left;
                    rect.Y += padding.Top;
                    rect.Width -= padding.Horizontal;
                    rect.Height -= padding.Vertical;
                }

                string textToDraw = this.Text;

                if (textToDraw.Length > RadGdiGraphics.GdiStringLengthLimit)
                {
                    textToDraw = textToDraw.Substring(0, RadGdiGraphics.GdiStringLengthLimit);
                }

                try
                {
                    g.DrawString(textToDraw, this.Font, brush, rect, format);
                }
                catch 
                {
                    try
                    {
                        using (Font newFont = new Font(this.Font, FontStyle.Regular))
                        {
                            g.DrawString(textToDraw, newFont, brush, rect, format);
                        }
                    }
                    catch
                    {
                        g.DrawString(textToDraw, Control.DefaultFont, brush, rect, format);
                    }
                }
            }
        }
    }
}
